##############################
##2013 Prefresher
##Day 2 - Working with Data in R
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################

## This file works through some basics of working
## with vectors, matrices, dataframes, and lists in R


#######################################################
## Working Directories
#######################################################

## The most common thing we want to do in R is work with data of some sort.
## Usually these data are in some format like .csv (excel spreadsheets),
## .dta (Stata dataset), or .txt (text files).

## Reading these into R (i.e. adding them to your workspace) is very simple,
## but it first requires telling R where they are on your computer.

## We do this by setting our working directory, which is the folder in which
## the files are stored. If you try to load data into R which is not inside
## the folder which is specified in your working directory, you'll get an error.
## Also, anytime you export anything from R (new datasets, graphs, etc.) it will
## save them to your working directory

## To see your current working directory, use the getwd() function
getwd()

## I want to change the working directory to my "R tutorials" folder, so
## I'll use the setwd() function

setwd("c:/users/stephen/dropbox/gov prefresher/2014 prefresher/r tutorials/day 2 data")

## note that I wrapped the directory path in quotation marks
## If you don't do that, R will assume you're referring to an object in your
## workspace in R, not a folder on your computer


## Another useful way to find a directory or file is to use file.choose()
setwd(file.choose())

## I think you can do something similar with a Mac using "cmd + D"


## If you want to check out what files are inside of your working directory, 
## use list.files()

list.files()











########################################################
##### Loading data into R
########################################################

## There are three basic filetypes that you typically would want to
## read into R: .RData (which is data created by R),
## .csv or .txt (which is data stored in a table format, maybe created by Excel),
## and everything else (Stata, SAS, SPSS, shapefiles, etc.).

## The easiest data to load into R is RData:

load("day2data.RData")

## You'll see now that our workspace has all the same objects
## as we ended with yesterday.
ls()


## The second type of data that's common to read are tables and
## comma separated values (.csv).

pres <- read.csv("pres_returns.csv")






## Often we want to read in data that's in a different format, like
## Stata or SPSS.

## The function we use to read Stata files (.dta) is called read.dta()
## Let's try to add the pres08_ga.dta file to our workspace

pres <- read.dta("pres08_ga.dta")
## It didn't work! That's because R doesn't know what the read.dta function is!



## To be able to read data in these formats, we have to first load
## the "foreign" package.
## Packages in R are user generated, and provide additional functionality
## and customizability to what you can do with R.

## The first time you want to use a package, use the install.packages() function:
install.packages("foreign")


## Once a package is installed, you have to load it into R each time you start
## a new R session.
## Do this by using their the library() function or the require() function:

library(foreign)
## or
require(foreign)


## Now let's try again to load the presidential election data
pres <- read.dta("pres08_ga.dta")

## It worked and it stored it as a dataframe!
pres
is.data.frame(pres)




#################################################################
## Working with objects and exploring data
#################################################################


##########################
### Vectors
##########################

## The concatenate function, c(), can be used to combine vectors together:

vector1
vector2 <- 10:14
vector2

comb.vect <- c(vector1, vector2)
comb.vect


## You can also use R to add/subtract vectors:
sum.vect <- vector1 + vector2
sum.vect

diff.vect <- vector1 - vector2
diff.vect


## And multiply/divide them...

vector1 * 3

vector1 / 3

## Note that the function below is not vector multiplication or dot products.
## Instead, R multiplies or divides the first element by the first element,
## the second by the second, etc.
vector1 * vector2

vector1 / vector2


## If we do want the dot product of two vectors, use the %*% operator:
vector1 %*% vector2


## We can also use the basic functions in R to get information about our
## vectors. The length function tells us how many elements a vector has:
length(vector1)


## Putting brackets, [], at the end of an object can pull out specific elements
## of that object.
## Let's say we want to get the third element of a particular vector:

vector1[3]


## We can even use brackets to reassign values within an object, without
## having to make a totally new object:
## We see that the third element of vector1 equals 6. Let's change is to -13

vector1[3] <- -13

vector1



## We can even put functions inside of brackets. What if we want to get the last
## element of a vector, but we don't know exactly how long the vector is?

vector1[length(vector1)] ## It's like magic!



## What if we want everything except the 4th element of the vector?
## (Anybody familiar with Python or other computing languages should note that
## this syntax is unique to R)
vector1[-4]


## Write a line of code that pulls out the first and third elements of vector1,
## and finds the standard deviation of those two numbers.
## Hint: we can put other functions inside of the brackets.
## Hint: use the ?? function to figure out how to get a standard deviation













## Answer
sd(vector1[c(1,3)])


###########################################
##### Matrix manipulations and matric algebra
###########################################

## If we have two vectors that we want to combine together into a matrix,
## we can use the column bind, cbind(), function

c.matrix <- cbind(vector1, vector2)

is.matrix(c.matrix) ## It's a matrix now!


## We can use the row bind, rbind(), function to bind vectors together by row:

r.matrix <- rbind(vector1, vector2)



## Just like vectors, we can do algebraic manipulations of matrices:
c.matrix + 2

c.matrix - 2

c.matrix * 2

c.matrix / 2



## We can do lots of other operations to matrices:
## We can get the dimensionality of the matrix
dim(c.matrix)


## Or, if we only want the number of rows in the matrix:
nrow(c.matrix)


## Or just the number of columns
ncol(c.matrix)



## We can transpose matrices:
t(c.matrix)



## We can add and substract them:

c.matrix + c.matrix

c.matrix - c.matrix



## We can do matrix multiplication using the %*% operator

mult.matrix <- c.matrix %*% r.matrix

mult.matrix



## We can get the determinant of square matrices:

det(mult.matrix)



## We can do matrix inversion of square matrices using the solve() function:

solve(mult.matrix)
# The function above doesn't work! But why? Ideas?















## Answer: The error says the "system is computationally singular." In math-speak,
## singular means that the matrix is not of "full rank" and the determinant
## is zero. And when you invert a matrix you have to divide by zero.

## But we calculated the determinant, and it wasn't zero!

## It wasn't exactly zero, but was 3.7 x 10^(-38). That's super, super close
## to zero. Computers aren't perfect, so with rounding errors and other
## computing issues, R treats it as basically zero, and freaks out.



## Let's look at an example of a matrix that is invertible

matrix1 <- matrix(c(1,6,4,3), nrow = 2, ncol = 2)
matrix1

solve(matrix1)




## Another useful function for matrices is extracting the diagonal.
## This is particularly useful when you do regression or maximum likelihood
## and need to extract the standard errors.

matrix1
diag(matrix1)

mult.matrix
diag(mult.matrix)



## We can even assign values to the diagonal of a matrix
diag(matrix1) <- c(1,1)
matrix1





## Earlier we talked about how you can use brackets, [], to extract elements
## of a vector. We can do the same with matrices.
## Remember that vectors are one dimensional, while matrices are two
## dimensional.
## To extract an element from a matrix use the follow structure of your code:
## matrix[row, column], where the row number and column numbers are called
## indexes.

## So if we want to see the third row, fourth column of mult.matrix:
mult.matrix[3,4]
mult.matrix


## If we leave the row number blank, but specify a column, R will give us
## a vector of all the values in that column
mult.matrix[,4]


## What happens if we forget the comma?
## R outputs the 4th element of the matrix.
## This is a really common mistake. Don't let it screw you up
mult.matrix[4]





## What if we want to name the columns of our matrix?
## We can use the colnames() function to extract the names.
## You can do the same thing with the rownames() function.

colnames(mult.matrix) ## there's no column names yet!

## We can assign them assigning a character vector
colnames(mult.matrix) <- c("var1",
                           "var2",
                           "var3",
                           "var4",
                           "var6")

colnames(mult.matrix)
mult.matrix

## I made a mistake in naming one of the columns!
## Write a line of code that renames var6 to var5.
## Try to do this without touching any of the other variable names.













## The two lines of code do the same thing, although the second one
## is slightly more general, in the case that you didn't know exactly how
## many columns the matrix has (you just knew you wanted to change
## the name of last one).

colnames(mult.matrix)[5] <- "var5"

colnames(mult.matrix)[ncol(mult.matrix)] <- "var5"

mult.matrix



###############################################
## Dataframes
###############################################

## Earlier we loaded in a dataset of presidential election data.
## Let's give it a look.

pres

## We can just look at the first few rows of the dataset by using head().
head(pres)


## Recall that the pres object is a dataframe. We can get the variable names
## of the dataframe by using colnames() or names()

colnames(pres)

names(pres)




## Like matrices, we can extract specific values of a dataframe using brackets

pres[1,3]



## Unlike matrices, we can get whole variables from a dataframe by using
## its variable name with the $ operator

pres[,7]

pres$dempct

pres[,"dempct"]




## We can also see which unique values a particular variable takes using unique()

unique(pres$year)

## The unique() function also works for vectors:

unique(c(2,1,1,1,1,1,1,1,2,3,4,4,5))





################################################
#### Manipulation of lists
################################################

## Extracting values from a list

ourlist

## Lists use double indexing by using double brackets, [[]]
## Get the first element of the list:

ourlist[[1]]


## Pull out the fifth element of the list:

ourlist[[5]]

## Let's check what kind of object this element is.
## Check this by using the class() function

class(ourlist[[5]])


## Because outlist[[5]] is a matrix, we can do any sort of matrix
## manipulations to do:

#transpose:
t(ourlist[[5]])


asdf <- ourlist[[5]]
asdf
